/*
 * ttable.h
 *
 * 8/6/93    Added indexStoreFlag and myIndices fields to DSTABLE - mustafa
 * 5/23/93   Added regular distributions into translation table code - mustafa
 */
#ifndef __TTABLE_H_
#define __TTABLE_H_

#define MAXCHUNK    __MAXCHUNK
#define LOCAL       0
#define DIRECT      1
#define PARTIAL     2
#define NONLOCAL    3
#define HIT         4
#define TT_SIZE     1000

#define DISTRIBUTED   0
#define REGULAR       1

#define BLOCKED       1
#define CYCLIC        2

#define ON            1
#define OFF           0

typedef struct DsChunk {
  int *Proc,*Offset;
} PAGE, *PAGEPTR;


typedef int *intPtr;

/* 
 * structure for distributed translation table 
 */
typedef struct DsTable {
  int  pageSize;       /* Size of a single page in the translation table */ 
  int  LastBlock;      /* Remainder in the block distribution of pages */
  int  Replication;    /* Number of blocks replicated in the processor */
  int  numPages;       /* Total number of pages in the table */
  int *Loffset;        /* Buffer space for the storage of offsets */
  int *Lproc;          /* Buffer space for the storage of procs */
  int  nMyIndices;     /* size of portion of translation table allocated to my indices */
  float repFactor;     /* Replication factor */
  PAGEPTR *pageTable;  /* Pointer to the chunks stored in the table */
} DSTABLE, *DSTABLEPTR;



/*
 * Structure for regular translation, which does not need enumeration
 */
typedef struct regTable {
  int   dist_type ;
  int   data_size ;
  int   block_size ;
}  REG_TABLE, *REGTABLEPTR ;


/*
 * Structure for User Translation Table
 */
typedef struct trans_table {
  int             type;
  DSTABLE        *tabptr; 
  REG_TABLE      *regptr;
} TTABLE, *TTABLEPTR;



/* 
 * Function prototypes
 */
DSTABLE *BuildDistTable();
DSTABLE *InitDistTable();
int      DrefDistTable();
void     ShuffleDistTable();
void     DestroyDistTable();
void     regdereference();
float    tableGetReplication();
int      tableGetBlockSize();
intPtr   tableGetIndices();
int      tableGetIndexCount();
TTABLE   *build_reg_translation_table ();
TTABLE   *mkDistTable();
TTABLE   *build_translation_table();
void     free_table();
void     dereference();
TTABLE   *init_ttable_with_proc();
int      trans_table_size();
int      *trans_table_color();

PAGEPTR  freePage();
#endif



